import tkinter as tk
import tkinter.ttk as ttk

from aui.widgets import Input
from aui_tk.widgets.Base import Base
from aui_tk import Errors


class Input(Input, Base):
    """Represents Tk entry"""

    def __init__(self, value="", onenter=None, **kwargs):
        """
        Keyword arguments:
        value       -- default value
                       | str (default: "")
        onenter     -- function called after the return button is pressed
                       | function: Input -> void
        Arguments:
        **kwargs    -- any keyword arguments that will be pased
                       additionally when creating ttk.Entry
        Attributes:
        wide        -- makes the input wide
        """
        self._initial_value = value
        self._value = None
        self.onenter = onenter
        self._extra_params = kwargs
        self._input_field = None
        self.__wide = False

    def __make_wide(self):
        self.__wide = True

    def __getattr__(self, attr):
        functions = {
            'wide': self.__make_wide
        }

        if attr not in functions:
            raise KeyError("`{}` is not a valid button modifier".format(attr))

        functions[attr]()
        return self

    @property
    def value(self):
        """Get input's value"""
        try:
            return self._value.get()
        except:
            raise Errors.NotInitializedException("Input has not been initialized yet")

    @value.setter
    def value(self, new_value):
        """Set input's value"""
        try:
            self._value.set(new_value)
        except:
            raise Errors.NotInitializedException("Input has not been initialized yet")

    def __onenter(self, event):
        """Calls self.onenter function. Bound to <Return> press."""
        if self.onenter:
            self.onenter(self)

    def destroy(self):
        """Destroys the input field"""
        self._input_field.destroy()

    def ctrl_a_handler(self, e):
        e.widget.select_range(0, tk.constants.END)
        return 'break'

    def create(self, parent, align=None):
        """
        Creates input and assigns it to its parent

        Keyword arguments:
        parent -- parent of the element to be put into
        align  -- alignment of the element in container (UP/RIGHT/BOTTOM/LEFT)
        """
        self._value = tk.StringVar()
        self._value.set(self._initial_value)

        self._input_field = ttk.Entry(parent,
                                      textvariable=self._value,
                                      **self._extra_params)
        self._input_field.pack(side=align,
                               fill=tk.constants.X if self.__wide else None)
        self._input_field.bind('<Control-a>', self.ctrl_a_handler)
        self._input_field.bind('<Return>', self.__onenter)
